@@ -30,6 +30,10 @@ from The Open Group.
  * Author:  Davor Matic, MIT X Consortium
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include <X11/Xaw/XawInit.h>
@@ -41,6 +45,7 @@ from The Open Group.
 #include "BitmapP.h"
 #include "Bitmap.h"
 
+#include <assert.h>
 #include <stdio.h>
 #include <math.h>
 
@@ -389,8 +394,10 @@ BWGetUnzoomedPixmap(Widget w)
 			    BW->bitmap.zoom.image->width,
 			    BW->bitmap.zoom.image->height, 1);
 	if (!(gc = XCreateGC(XtDisplay(w), pix,
-			     (unsigned long) 0, (XGCValues *) 0)))
+			     (unsigned long) 0, (XGCValues *) 0))) {
+	    XFreePixmap(XtDisplay(w), pix);
 	    return (Pixmap) None;
+	}
 
 	XPutImage(XtDisplay(w), pix, gc,
 		  BW->bitmap.zoom.image,
@@ -410,8 +417,10 @@ BWGetUnzoomedPixmap(Widget w)
 			    BW->bitmap.image->width,
 			    BW->bitmap.image->height, 1);
 	if (! (gc = XCreateGC(XtDisplay(w), pix,
-			      (unsigned long) 0, (XGCValues *) 0)))
+			      (unsigned long) 0, (XGCValues *) 0))){
+	    XFreePixmap(XtDisplay(w), pix);
 	    return (Pixmap) None;
+	}
 
 	XPutImage(XtDisplay(w), pix, gc,
 		  BW->bitmap.image,
@@ -643,7 +652,7 @@ BWPutImage(BitmapWidget w, Display *display, Drawable drawable, GC gc,
 }
 #endif
 
-static String
+static char *
 StripFilename(_Xconst _XtString filename)
 {
     const char *begin = strrchr(filename, '/');
@@ -682,10 +691,14 @@ XmuWriteBitmapDataToFile(_Xconst _XtString filename,
     else
     	file = fopen(filename, "w+");
 
-    if (!basename || !strcmp(basename, "") || !strcmp(basename, "-"))
-	basename = StripFilename(filename);
-
     if (file) {
+	char *new_basename;
+
+	if (!basename || !strcmp(basename, "") || !strcmp(basename, "-"))
+	    basename = new_basename = StripFilename(filename);
+	else
+	    new_basename = NULL;
+
 	fprintf(file, "#define %s_width %d\n", basename, width);
 	fprintf(file, "#define %s_height %d\n", basename, height);
 	if (QuerySet(x_hot, y_hot)) {
@@ -704,6 +717,7 @@ XmuWriteBitmapDataToFile(_Xconst _XtString filename,
 	if (file != stdout)
 	    fclose(file);
 
+	XtFree(new_basename);
 	return BitmapSuccess;
     }
 
@@ -898,20 +912,20 @@ Initialize(Widget wrequest, Widget wnew, ArgList argv, Cardinal *argc)
     {
 	int status;
 	XImage *image, *buffer;
-	unsigned char *image_data;
-	char *buffer_data;
+	unsigned char *image_data2;
+	char *buffer_data2;
 	unsigned int width, height;
 	int x_hot, y_hot;
 
 	status = XmuReadBitmapDataFromFile(new->bitmap.filename,
-					   &width, &height, &image_data,
+					   &width, &height, &image_data2,
 					   &x_hot, &y_hot);
 	if (status == BitmapSuccess) {
 
-	    buffer_data = CreateCleanData(Length(width, height));
+	    buffer_data2 = CreateCleanData(Length(width, height));
 
-	    image = CreateBitmapImage(new, (char *)image_data, width, height);
-	    buffer = CreateBitmapImage(new, buffer_data, width, height);
+	    image = CreateBitmapImage(new, (char *)image_data2, width, height);
+	    buffer = CreateBitmapImage(new, buffer_data2, width, height);
 
 	    TransferImageData(new->bitmap.image, buffer);
 
@@ -953,12 +967,14 @@ BWParseSize(String size, Dimension *width, Dimension *height)
 
   status = XParseGeometry(size, &x, &y, &w, &h);
 
-  if (status & (WidthValue | HeightValue)) {
+  if (status & WidthValue) {
     *width = (Dimension) w;
-    *height = (Dimension) h;
-    return True;
+    if (status & HeightValue) {
+      *height = (Dimension) h;
+      return True;
+    }
   }
-  else return False;
+  return False;
 
 }
 
@@ -1098,7 +1114,7 @@ BWReadFile(Widget w, _Xconst _XtString filename, _Xconst _XtString basename) /*
 	XtFree(BW->bitmap.filename);
 	BW->bitmap.filename = XtNewString(filename);
 	XtFree(BW->bitmap.basename);
-	BW->bitmap.basename= XtNewString(StripFilename(filename));
+	BW->bitmap.basename = StripFilename(filename);
 
 	BWUnmark(w);
 
@@ -1194,7 +1210,7 @@ BWWriteFile(Widget w, _Xconst _XtString filename, _Xconst _XtString basename)
 	XtFree(BW->bitmap.filename);
 	BW->bitmap.filename = XtNewString(filename);
 	XtFree(BW->bitmap.basename);
-	BW->bitmap.basename= XtNewString(StripFilename(filename));
+	BW->bitmap.basename = StripFilename(filename);
     }
     if (!basename) basename = BW->bitmap.basename;
     else {
@@ -1217,8 +1233,8 @@ BWWriteFile(Widget w, _Xconst _XtString filename, _Xconst _XtString basename)
     return status;
 }
 
-String
-BWGetFilename(Widget w, String *str)
+_XtString
+BWGetFilename(Widget w, _XtString *str)
 {
     BitmapWidget BW = (BitmapWidget) w;
 
@@ -1227,13 +1243,15 @@ BWGetFilename(Widget w, String *str)
     return *str;
 }
 
-String
-BWGetFilepath(Widget w, String *str)
+_XtString
+BWGetFilepath(Widget w, _XtString *str)
 {
     BitmapWidget BW = (BitmapWidget) w;
-    String end;
+    _XtString end;
 
     *str = XtNewString(BW->bitmap.filename);
+    assert(*str);
+
     end = strrchr(*str, '/');
 
     if (end)
@@ -1245,8 +1263,8 @@ BWGetFilepath(Widget w, String *str)
 }
 
 
-String
-BWGetBasename(Widget w, String *str)
+_XtString
+BWGetBasename(Widget w, _XtString *str)
 {
     BitmapWidget BW = (BitmapWidget) w;
 
@@ -1377,7 +1395,7 @@ BWZoomIn(Widget w,
     BW->bitmap.mark.to_x = NotSet;
     BW->bitmap.mark.to_y = NotSet;
     BW->bitmap.zooming = True;
-    BW->bitmap.grid = True; /* potencially true, could use a resource here */
+    BW->bitmap.grid = True; /* potentially true, could use a resource here */
 
     FixHotSpot(BW);
 
